import { Keybinding, ResolvedKeybinding } from "@codingame/monaco-vscode-api/vscode/vs/base/common/keybindings";
import { ICommandService } from "@codingame/monaco-vscode-api/vscode/vs/platform/commands/common/commands.service";
import { IContextKeyService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service";
import { IFileService } from "@codingame/monaco-vscode-api/vscode/vs/platform/files/common/files.service";
import { AbstractKeybindingService } from "@codingame/monaco-vscode-api/vscode/vs/platform/keybinding/common/abstractKeybindingService";
import { IKeyboardEvent, KeybindingsSchemaContribution } from "@codingame/monaco-vscode-api/vscode/vs/platform/keybinding/common/keybinding";
import { KeybindingResolver } from "@codingame/monaco-vscode-api/vscode/vs/platform/keybinding/common/keybindingResolver";
import { ResolvedKeybindingItem } from "@codingame/monaco-vscode-api/vscode/vs/platform/keybinding/common/resolvedKeybindingItem";
import { IKeyboardLayoutService } from "@codingame/monaco-vscode-api/vscode/vs/platform/keyboardLayout/common/keyboardLayout.service";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { INotificationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification.service";
import { ITelemetryService } from "@codingame/monaco-vscode-api/vscode/vs/platform/telemetry/common/telemetry.service";
import { IUriIdentityService } from "@codingame/monaco-vscode-api/vscode/vs/platform/uriIdentity/common/uriIdentity.service";
import { IExtensionService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/extensions/common/extensions.service";
import { IHostService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/host/browser/host.service";
import { IUserDataProfileService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/userDataProfile/common/userDataProfile.service";
import { ILayoutService } from "@codingame/monaco-vscode-api/vscode/vs/platform/layout/browser/layoutService.service";
export declare class WorkbenchKeybindingService extends AbstractKeybindingService {
    private readonly hostService;
    private readonly keyboardLayoutService;
    private readonly layoutService;
    private _keyboardMapper;
    private _cachedResolver;
    private userKeybindings;
    private isComposingGlobalContextKey;
    private _keybindingHoldMode;
    private readonly _contributions;
    private readonly kbsJsonSchema;
    constructor(contextKeyService: IContextKeyService, commandService: ICommandService, telemetryService: ITelemetryService, notificationService: INotificationService, userDataProfileService: IUserDataProfileService, hostService: IHostService, extensionService: IExtensionService, fileService: IFileService, uriIdentityService: IUriIdentityService, logService: ILogService, keyboardLayoutService: IKeyboardLayoutService, layoutService: ILayoutService);
    private _registerKeyListeners;
    registerSchemaContribution(contribution: KeybindingsSchemaContribution): void;
    private updateKeybindingsJsonSchema;
    private _printKeybinding;
    private _printResolvedKeybinding;
    private _printResolvedKeybindings;
    private _dumpResolveKeybindingDebugInfo;
    _dumpDebugInfo(): string;
    _dumpDebugInfoJSON(): string;
    enableKeybindingHoldMode(commandId: string): Promise<void> | undefined;
    private _resetKeybindingHoldMode;
    customKeybindingsCount(): number;
    protected updateResolver(): void;
    protected getUserKeybindingItems(): ResolvedKeybindingItem[];
    protected _getResolver(): KeybindingResolver;
    protected _documentHasFocus(): boolean;
    private _resolveKeybindingItems;
    private _resolveUserKeybindingItems;
    private _assertBrowserConflicts;
    resolveKeybinding(kb: Keybinding): ResolvedKeybinding[];
    resolveKeyboardEvent(keyboardEvent: IKeyboardEvent): ResolvedKeybinding;
    resolveUserBinding(userBinding: string): ResolvedKeybinding[];
    private _handleKeybindingsExtensionPointUser;
    private _handleKeybinding;
    private static bindToCurrentPlatform;
    private _asCommandRule;
    getDefaultKeybindingsContent(): string;
    private static _getDefaultKeybindings;
    private static _getAllCommandsAsComment;
    mightProducePrintableCharacter(event: IKeyboardEvent): boolean;
}
